require 'uri'
require 'net/http'

module Wild
  module View
    # RemoteRenderer
    #
    # This class extend ActionView::Renderer and overwriten the render methods
    # to download views from configure endpoint
    class RemoteRenderer < ActionView::Renderer
      def render(context, options)
        options.delete(:remote)
        super(context, options)
      end

      # def render_body(context, options)
      #   super(context, options)
      # end
      #
      # def render_template(context, options)
      #   super(context, options)
      # end

      def render_partial(context, options, &block) #:nodoc:
        download_and_save_view(options[:partial])
        super(context, options)
      end

      private

      def download_and_save_view(view_name)
        view_data = download_view(view_name)
        save_view(view_name, view_data)
      end

      def download_view(view_name)
        response = http.get endpoint_path(view_name)
        response.value
        response.body
      end

      def save_view(view_name, view_data)
        view_path = build_view_path(view_name)
        ensure_directory_exists(view_path)
        write_view(view_path, view_data)
      end

      def build_view_path(view_name)
        path = view_name.split('/')
        path[-1] = view_file_name(path.last)
        Wild::Views.root.join(*path)
      end

      def view_file_name(view_name)
        "_#{view_name}.html.erb"
      end

      def endpoint_path(view_name)
        "/#{view_name}"
      end


      def http
        @http ||= begin
          uri = URI(Wild::Views.config['endpoint'])
          Net::HTTP.start(uri.host, uri.port, use_ssl: uri.scheme == 'https')
        end
      end

      def ensure_directory_exists(view_name)
        FileUtils.mkdir_p(File.dirname(view_name))
      end

      def write_view(view_path, view_data)
        File.open(view_path, 'w') { |f| f.write(view_data) }
      end
    end
  end
end
